@@ -3,10 +3,10 @@ package maxminddb
 import (
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"math/big"
 	"math/rand"
 	"net"
+	"os"
 	"path/filepath"
 	"testing"
 	"time"
@@ -18,7 +18,11 @@ import (
 func TestReader(t *testing.T) {
 	for _, recordSize := range []uint{24, 28, 32} {
 		for _, ipVersion := range []uint{4, 6} {
-			fileName := fmt.Sprintf(testFile("MaxMind-DB-test-ipv%d-%d.mmdb"), ipVersion, recordSize)
+			fileName := fmt.Sprintf(
+				testFile("MaxMind-DB-test-ipv%d-%d.mmdb"),
+				ipVersion,
+				recordSize,
+			)
 			reader, err := Open(fileName)
 			require.NoError(t, err, "unexpected error while opening database: %v", err)
 			checkMetadata(t, reader, ipVersion, recordSize)
@@ -35,8 +39,13 @@ func TestReader(t *testing.T) {
 func TestReaderBytes(t *testing.T) {
 	for _, recordSize := range []uint{24, 28, 32} {
 		for _, ipVersion := range []uint{4, 6} {
-			fileName := fmt.Sprintf(testFile("MaxMind-DB-test-ipv%d-%d.mmdb"), ipVersion, recordSize)
-			bytes, _ := ioutil.ReadFile(fileName)
+			fileName := fmt.Sprintf(
+				testFile("MaxMind-DB-test-ipv%d-%d.mmdb"),
+				ipVersion,
+				recordSize,
+			)
+			bytes, err := os.ReadFile(fileName)
+			require.NoError(t, err)
 			reader, err := FromBytes(bytes)
 			require.NoError(t, err, "unexpected error while opening bytes: %v", err)
 
@@ -54,8 +63,8 @@ func TestReaderBytes(t *testing.T) {
 func TestLookupNetwork(t *testing.T) {
 	bigInt := new(big.Int)
 	bigInt.SetString("1329227995784915872903807060280344576", 10)
-	decoderRecord := map[string]interface{}{
-		"array": []interface{}{
+	decoderRecord := map[string]any{
+		"array": []any{
 			uint64(1),
 			uint64(2),
 			uint64(3),
@@ -70,9 +79,9 @@ func TestLookupNetwork(t *testing.T) {
 		"double": 42.123456,
 		"float":  float32(1.1),
 		"int32":  -268435456,
-		"map": map[string]interface{}{
-			"mapX": map[string]interface{}{
-				"arrayX": []interface{}{
+		"map": map[string]any{
+			"mapX": map[string]any{
+				"arrayX": []any{
 					uint64(0x7),
 					uint64(0x8),
 					uint64(0x9),
@@ -91,7 +100,7 @@ func TestLookupNetwork(t *testing.T) {
 		IP             net.IP
 		DBFile         string
 		ExpectedCIDR   string
-		ExpectedRecord interface{}
+		ExpectedRecord any
 		ExpectedOK     bool
 	}{
 		{
@@ -105,28 +114,28 @@ func TestLookupNetwork(t *testing.T) {
 			IP:             net.ParseIP("::1:ffff:ffff"),
 			DBFile:         "MaxMind-DB-test-ipv6-24.mmdb",
 			ExpectedCIDR:   "::1:ffff:ffff/128",
-			ExpectedRecord: map[string]interface{}{"ip": "::1:ffff:ffff"},
+			ExpectedRecord: map[string]any{"ip": "::1:ffff:ffff"},
 			ExpectedOK:     true,
 		},
 		{
 			IP:             net.ParseIP("::2:0:1"),
 			DBFile:         "MaxMind-DB-test-ipv6-24.mmdb",
 			ExpectedCIDR:   "::2:0:0/122",
-			ExpectedRecord: map[string]interface{}{"ip": "::2:0:0"},
+			ExpectedRecord: map[string]any{"ip": "::2:0:0"},
 			ExpectedOK:     true,
 		},
 		{
 			IP:             net.ParseIP("1.1.1.1"),
 			DBFile:         "MaxMind-DB-test-ipv4-24.mmdb",
 			ExpectedCIDR:   "1.1.1.1/32",
-			ExpectedRecord: map[string]interface{}{"ip": "1.1.1.1"},
+			ExpectedRecord: map[string]any{"ip": "1.1.1.1"},
 			ExpectedOK:     true,
 		},
 		{
 			IP:             net.ParseIP("1.1.1.3"),
 			DBFile:         "MaxMind-DB-test-ipv4-24.mmdb",
 			ExpectedCIDR:   "1.1.1.2/31",
-			ExpectedRecord: map[string]interface{}{"ip": "1.1.1.2"},
+			ExpectedRecord: map[string]any{"ip": "1.1.1.2"},
 			ExpectedOK:     true,
 		},
 		{
@@ -182,7 +191,7 @@ func TestLookupNetwork(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(fmt.Sprintf("%s - %s", test.DBFile, test.IP), func(t *testing.T) {
-			var record interface{}
+			var record any
 			reader, err := Open(testFile(test.DBFile))
 			require.NoError(t, err)
 
@@ -199,25 +208,30 @@ func TestDecodingToInterface(t *testing.T) {
 	reader, err := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
 	require.NoError(t, err, "unexpected error while opening database: %v", err)
 
-	var recordInterface interface{}
+	var recordInterface any
 	err = reader.Lookup(net.ParseIP("::1.1.1.0"), &recordInterface)
 	require.NoError(t, err, "unexpected error while doing lookup: %v", err)
 
 	checkDecodingToInterface(t, recordInterface)
 }
 
-func checkDecodingToInterface(t *testing.T, recordInterface interface{}) {
-	record := recordInterface.(map[string]interface{})
-	assert.Equal(t, []interface{}{uint64(1), uint64(2), uint64(3)}, record["array"])
+func TestMetadataPointer(t *testing.T) {
+	_, err := Open(testFile("MaxMind-DB-test-metadata-pointers.mmdb"))
+	require.NoError(t, err, "unexpected error while opening database: %v", err)
+}
+
+func checkDecodingToInterface(t *testing.T, recordInterface any) {
+	record := recordInterface.(map[string]any)
+	assert.Equal(t, []any{uint64(1), uint64(2), uint64(3)}, record["array"])
 	assert.Equal(t, true, record["boolean"])
 	assert.Equal(t, []byte{0x00, 0x00, 0x00, 0x2a}, record["bytes"])
 	assert.Equal(t, 42.123456, record["double"])
 	assert.Equal(t, float32(1.1), record["float"])
 	assert.Equal(t, -268435456, record["int32"])
 	assert.Equal(t,
-		map[string]interface{}{
-			"mapX": map[string]interface{}{
-				"arrayX":       []interface{}{uint64(7), uint64(8), uint64(9)},
+		map[string]any{
+			"mapX": map[string]any{
+				"arrayX":       []any{uint64(7), uint64(8), uint64(9)},
 				"utf8_stringX": "hello",
 			},
 		},
@@ -233,20 +247,19 @@ func checkDecodingToInterface(t *testing.T, recordInterface interface{}) {
 	assert.Equal(t, bigInt, record["uint128"])
 }
 
-// nolint: maligned
 type TestType struct {
-	Array      []uint                 `maxminddb:"array"`
-	Boolean    bool                   `maxminddb:"boolean"`
-	Bytes      []byte                 `maxminddb:"bytes"`
-	Double     float64                `maxminddb:"double"`
-	Float      float32                `maxminddb:"float"`
-	Int32      int32                  `maxminddb:"int32"`
-	Map        map[string]interface{} `maxminddb:"map"`
-	Uint16     uint16                 `maxminddb:"uint16"`
-	Uint32     uint32                 `maxminddb:"uint32"`
-	Uint64     uint64                 `maxminddb:"uint64"`
-	Uint128    big.Int                `maxminddb:"uint128"`
-	Utf8String string                 `maxminddb:"utf8_string"`
+	Array      []uint         `maxminddb:"array"`
+	Boolean    bool           `maxminddb:"boolean"`
+	Bytes      []byte         `maxminddb:"bytes"`
+	Double     float64        `maxminddb:"double"`
+	Float      float32        `maxminddb:"float"`
+	Int32      int32          `maxminddb:"int32"`
+	Map        map[string]any `maxminddb:"map"`
+	Uint16     uint16         `maxminddb:"uint16"`
+	Uint32     uint32         `maxminddb:"uint32"`
+	Uint64     uint64         `maxminddb:"uint64"`
+	Uint128    big.Int        `maxminddb:"uint128"`
+	Utf8String string         `maxminddb:"utf8_string"`
 }
 
 func TestDecoder(t *testing.T) {
@@ -262,9 +275,9 @@ func TestDecoder(t *testing.T) {
 		assert.Equal(t, int32(-268435456), result.Int32)
 
 		assert.Equal(t,
-			map[string]interface{}{
-				"mapX": map[string]interface{}{
-					"arrayX":       []interface{}{uint64(7), uint64(8), uint64(9)},
+			map[string]any{
+				"mapX": map[string]any{
+					"arrayX":       []any{uint64(7), uint64(8), uint64(9)},
 					"utf8_stringX": "hello",
 				},
 			},
@@ -326,7 +339,11 @@ func TestNonEmptyNilInterface(t *testing.T) {
 	require.NoError(t, err)
 
 	err = reader.Lookup(net.ParseIP("::1.1.1.0"), &result)
-	assert.Equal(t, "maxminddb: cannot unmarshal map into type maxminddb.TestInterface", err.Error())
+	assert.Equal(
+		t,
+		"maxminddb: cannot unmarshal map into type maxminddb.TestInterface",
+		err.Error(),
+	)
 }
 
 type CityTraits struct {
@@ -339,7 +356,7 @@ type City struct {
 
 func TestEmbeddedStructAsInterface(t *testing.T) {
 	var city City
-	var result interface{} = city.Traits
+	var result any = city.Traits
 
 	db, err := Open(testFile("GeoIP2-ISP-Test.mmdb"))
 	require.NoError(t, err)
@@ -437,6 +454,67 @@ func TestComplexStructWithNestingAndPointer(t *testing.T) {
 	assert.NoError(t, reader.Close())
 }
 
+// See GitHub #115.
+func TestNestedMapDecode(t *testing.T) {
+	db, err := Open(testFile("GeoIP2-Country-Test.mmdb"))
+	require.NoError(t, err)
+
+	var r map[string]map[string]any
+
+	require.NoError(t, db.Lookup(net.ParseIP("89.160.20.128"), &r))
+
+	assert.Equal(
+		t,
+		map[string]map[string]any{
+			"continent": {
+				"code":       "EU",
+				"geoname_id": uint64(6255148),
+				"names": map[string]any{
+					"de":    "Europa",
+					"en":    "Europe",
+					"es":    "Europa",
+					"fr":    "Europe",
+					"ja":    "ヨーロッパ",
+					"pt-BR": "Europa",
+					"ru":    "Европа",
+					"zh-CN": "欧洲",
+				},
+			},
+			"country": {
+				"geoname_id":           uint64(2661886),
+				"is_in_european_union": true,
+				"iso_code":             "SE",
+				"names": map[string]any{
+					"de":    "Schweden",
+					"en":    "Sweden",
+					"es":    "Suecia",
+					"fr":    "Suède",
+					"ja":    "スウェーデン王国",
+					"pt-BR": "Suécia",
+					"ru":    "Швеция",
+					"zh-CN": "瑞典",
+				},
+			},
+			"registered_country": {
+				"geoname_id":           uint64(2921044),
+				"is_in_european_union": true,
+				"iso_code":             "DE",
+				"names": map[string]any{
+					"de":    "Deutschland",
+					"en":    "Germany",
+					"es":    "Alemania",
+					"fr":    "Allemagne",
+					"ja":    "ドイツ連邦共和国",
+					"pt-BR": "Alemanha",
+					"ru":    "Германия",
+					"zh-CN": "德国",
+				},
+			},
+		},
+		r,
+	)
+}
+
 func TestNestedOffsetDecode(t *testing.T) {
 	db, err := Open(testFile("GeoIP2-City-Test.mmdb"))
 	require.NoError(t, err)
@@ -499,7 +577,9 @@ func TestIpv6inIpv4(t *testing.T) {
 	var emptyResult TestType
 	assert.Equal(t, emptyResult, result)
 
-	expected := errors.New("error looking up '2001::': you attempted to look up an IPv6 address in an IPv4-only database")
+	expected := errors.New(
+		"error looking up '2001::': you attempted to look up an IPv6 address in an IPv4-only database",
+	)
 	assert.Equal(t, expected, err)
 	assert.NoError(t, reader.Close(), "error on close")
 }
@@ -508,10 +588,12 @@ func TestBrokenDoubleDatabase(t *testing.T) {
 	reader, err := Open(testFile("GeoIP2-City-Test-Broken-Double-Format.mmdb"))
 	require.NoError(t, err, "unexpected error while opening database: %v", err)
 
-	var result interface{}
+	var result any
 	err = reader.Lookup(net.ParseIP("2001:220::"), &result)
 
-	expected := newInvalidDatabaseError("the MaxMind DB file's data section contains bad data (float 64 size of 2)")
+	expected := newInvalidDatabaseError(
+		"the MaxMind DB file's data section contains bad data (float 64 size of 2)",
+	)
 	assert.Equal(t, expected, err)
 	assert.NoError(t, reader.Close(), "error on close")
 }
@@ -536,30 +618,33 @@ func TestNonDatabase(t *testing.T) {
 }
 
 func TestDecodingToNonPointer(t *testing.T) {
-	reader, _ := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
+	reader, err := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
+	require.NoError(t, err)
 
-	var recordInterface interface{}
-	err := reader.Lookup(net.ParseIP("::1.1.1.0"), recordInterface)
+	var recordInterface any
+	err = reader.Lookup(net.ParseIP("::1.1.1.0"), recordInterface)
 	assert.Equal(t, "result param must be a pointer", err.Error())
 	assert.NoError(t, reader.Close(), "error on close")
 }
 
 func TestNilLookup(t *testing.T) {
-	reader, _ := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
+	reader, err := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
+	require.NoError(t, err)
 
-	var recordInterface interface{}
-	err := reader.Lookup(nil, recordInterface)
+	var recordInterface any
+	err = reader.Lookup(nil, recordInterface)
 	assert.Equal(t, "IP passed to Lookup cannot be nil", err.Error())
 	assert.NoError(t, reader.Close(), "error on close")
 }
 
 func TestUsingClosedDatabase(t *testing.T) {
-	reader, _ := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
+	reader, err := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
+	require.NoError(t, err)
 	require.NoError(t, reader.Close())
 
-	var recordInterface interface{}
+	var recordInterface any
 
-	err := reader.Lookup(nil, recordInterface)
+	err = reader.Lookup(nil, recordInterface)
 	assert.Equal(t, "cannot call Lookup on a closed database", err.Error())
 
 	_, err = reader.LookupOffset(nil)
@@ -693,8 +778,9 @@ func BenchmarkInterfaceLookup(b *testing.B) {
 	db, err := Open("GeoLite2-City.mmdb")
 	require.NoError(b, err)
 
+	//nolint:gosec // this is a test
 	r := rand.New(rand.NewSource(time.Now().UnixNano()))
-	var result interface{}
+	var result any
 
 	ip := make(net.IP, 4)
 	for i := 0; i < b.N; i++ {
@@ -711,8 +797,9 @@ func BenchmarkInterfaceLookupNetwork(b *testing.B) {
 	db, err := Open("GeoLite2-City.mmdb")
 	require.NoError(b, err)
 
+	//nolint:gosec // this is a test
 	r := rand.New(rand.NewSource(time.Now().UnixNano()))
-	var result interface{}
+	var result any
 
 	ip := make(net.IP, 4)
 	for i := 0; i < b.N; i++ {
@@ -779,6 +866,7 @@ func BenchmarkCityLookup(b *testing.B) {
 	db, err := Open("GeoLite2-City.mmdb")
 	require.NoError(b, err)
 
+	//nolint:gosec // this is a test
 	r := rand.New(rand.NewSource(time.Now().UnixNano()))
 	var result fullCity
 
@@ -797,6 +885,7 @@ func BenchmarkCityLookupNetwork(b *testing.B) {
 	db, err := Open("GeoLite2-City.mmdb")
 	require.NoError(b, err)
 
+	//nolint:gosec // this is a test
 	r := rand.New(rand.NewSource(time.Now().UnixNano()))
 	var result fullCity
 
@@ -821,6 +910,7 @@ func BenchmarkCountryCode(b *testing.B) {
 		} `maxminddb:"country"`
 	}
 
+	//nolint:gosec // this is a test
 	r := rand.New(rand.NewSource(0))
 	var result MinCountry
 
@@ -844,5 +934,5 @@ func randomIPv4Address(r *rand.Rand, ip []byte) {
 }
 
 func testFile(file string) string {
-	return filepath.Join("test-data/test-data", file)
+	return filepath.Join("test-data", "test-data", file)
 }
